#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include "amcc.h"
#include <time.h>

/* ---------------------------------- */
/*      DSP register offsets          */
/* ---------------------------------- */

/* PCI Bus Operation Registers */

#define OMB1    0x00    /* outgoing mailbox register 1 */
#define OMB2    0x04    /* outgoing mailbox register 2 */
#define OMB3    0x08    /* outgoing mailbox register 3 */
#define OMB4    0x0c    /* outgoing mailbox register 4 */
#define IMB1    0x10    /* incoming mailbox register 1 */
#define IMB2    0x14    /* incoming mailbox register 2 */
#define IMB3    0x18    /* incoming mailbox register 3 */
#define IMB4    0x1c    /* incoming mailbox register 4 */
#define MBEF    0x34    /* mail-box empty/full status reg */

#define PCICMD  0x08
#define PCIADDR 0x04
#define PCIPAGE 0x00

#define READY		1

void host_fill_memory(int seed);
void host_fill_memory_mem(int seed);
int  host_validate_memory(int seed);
int  host_validate_memory_mem(int seed);
void host_data_integrity();
void host_data_integrity_mem();
void host_tf_test();
void host_tfmb_test();
void host_read_write_loop();
void host_memory_short_test();

extern int looping;

/* --- keyboard interrupt routine --- */
#ifdef __cplusplus
    #define __CPPARGS ...
#else
    #define __CPPARGS
#endif
void interrupt get_out(__CPPARGS); /*    interrupt prototype */



extern dword base_address_0;  /* PCI_CS_BASE_ADDRESS_0 - amcc */
extern dword base_address_1;	/* PCI_CS_BASE_ADDRESS_1 - memory */
extern dword base_address_2;	/* PCI_CS_BASE_ADDRESS_2 - pio */
extern dword base_address_3;	/* PCI_CS_BASE_ADDRESS_3 - cpld */
extern dword base_address_4;	/* PCI_CS_BASE_ADDRESS_4 - tmr */

void host_fill_memory(int seed)
{
 int i, j;
 unsigned long write_l_val;    /* 4 bytes */

 printf ("TEST NUMBER 2, PCI Full Memory Test\n");

 /* write random words */
 srand(seed);

 for ( i = 0; i < 256; i++ ){
   outpd ((word)(base_address_3 + PCIPAGE), i);

	for ( j = 0; j < 1024; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		write_l_val =  (unsigned long)(0xff & rand());
		write_l_val += (unsigned long)(0xff & rand()) << 8;
		write_l_val += (unsigned long)(0xff & rand()) << 16;
		write_l_val += (unsigned long)(0xff & rand()) << 24;
		outpd (base_address_1, write_l_val);
		}
	}
}

void host_fill_memory_mem(int seed)
{
 int i, j;
 unsigned long write_l_val;    /* 4 bytes */

 printf ("TEST NUMBER 2, PCI Full Memory Test\n");

 /* write random words */
 srand(seed);

 for ( i = 1; i < 256; i++ ){
   outpd ((word)(base_address_3 + PCIPAGE), i);

	for ( j = 0; j < 1024; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		write_l_val =  (unsigned long)(0xff & rand());
		write_l_val += (unsigned long)(0xff & rand()) << 8;
		write_l_val += (unsigned long)(0xff & rand()) << 16;
		write_l_val += (unsigned long)(0xff & rand()) << 24;
		outpd (base_address_1, write_l_val);
		}
	}
}



int host_validate_memory(int seed)
{
 int i, j;
 int fail;
 unsigned long read_l_val;  /* 4 bytes */
 unsigned long write_l_val;

 fail = 0;

 /* read back the random words */
 srand(seed);
 for ( i = 0; i < 256; i++ ){
 	outpd ((word)(base_address_3 + PCIPAGE), i);

 	for ( j = 0; j < 1024; j++ ){
 		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
 		write_l_val =  (unsigned long)(0xff & rand());
 		write_l_val += (unsigned long)(0xff & rand()) << 8;
 		write_l_val += (unsigned long)(0xff & rand()) << 16;
 		write_l_val += (unsigned long)(0xff & rand()) << 24;
 		read_l_val = inpd(base_address_1);
 		if ( (unsigned long) read_l_val != (unsigned long) write_l_val ){
         fail  = fail + 1;
  		 //	printf("write_l_val %lx read_l_val %lx i=%x j=%x\n",
    	  //			write_l_val,read_l_val,i,j);
         }
	  	}
 }

 return(fail);
}

int host_validate_memory_mem(int seed)
{
 int i, j;
 int fail;
 unsigned long read_l_val;  /* 4 bytes */
 unsigned long write_l_val;

 fail = 0;

 /* read back the random words */
 srand(seed);
 for ( i = 1; i < 256; i++ ){
 	outpd ((word)(base_address_3 + PCIPAGE), i);

 	for ( j = 0; j < 1024; j++ ){
 		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
 		write_l_val =  (unsigned long)(0xff & rand());
 		write_l_val += (unsigned long)(0xff & rand()) << 8;
 		write_l_val += (unsigned long)(0xff & rand()) << 16;
 		write_l_val += (unsigned long)(0xff & rand()) << 24;
 		read_l_val = inpd(base_address_1);
 		if ( (unsigned long) read_l_val != (unsigned long) write_l_val ){
         fail  = fail + 1;
  	  //		printf("write_l_val %lx read_l_val %lx i=%x j=%x\n",
     //				write_l_val,read_l_val,i,j);
         }
	  	}
 }

 return(fail);
}




void host_data_integrity()
{
 long i;     // 4 bytes
 char key;
 int j, jb;
 int fail;
 unsigned long write_l_val;
 unsigned long read_l_val;  /* 4 bytes */
 void interrupt (*oldfunc)(__CPPARGS); /* interrupt function pointer */

 long total_fails;
 FILE *stream;

 write_l_val = 1;

 /* open a file for update */
 stream = fopen("log", "w+");

 inpd( (base_address_0 + IMB1) );  // flush
 printf ("MBEF = %lx\n", inpd( (base_address_0 + MBEF))  );

 puts("Press <Shift><PrtSc> to terminate");

 /* save the old interrupt */
 oldfunc  = _dos_getvect(5);

 /* install interrupt handler */
 _dos_setvect(5,get_out);

 looping = 1;
 total_fails = 0;

 while (looping) {
   for ( j = 1 ; j < 9 ; j++ ){
      host_fill_memory(j);
      outpd ( base_address_0 + OMB1, j );
                             // write command

      while( ((inpd( (word)(base_address_0 + MBEF)) & 0x00030000L )
           != 0x00030000L ) && looping  ) ;
                  /* loop till
                     incoming mailbox IMB1 (low 2 bytes) are full */

      jb =  0xffff & inpd((word)(base_address_0 + IMB1));
      printf("read back %d\n", jb);

      if ( fail = host_validate_memory(jb) ){
	 	   printf ("test fails, number of failures = %d\n", fail);
  	 	   fprintf (stream,"test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
	 	   printf ("test passes, errors(%d)\n", total_fails);
	 	   fprintf (stream,"test passes, errors(%d)\n", total_fails);
         }
      }
 }  /* while (looping) */

 /* restore to original interrupt routine */
 _dos_setvect(5,oldfunc);


 /* close the file */
 fclose(stream);


}


void host_data_integrity_mem()
{
 long i;     // 4 bytes
 char key;
 int j, jb;
 int fail;
 long total_fails;
 unsigned long write_l_val;
 unsigned long read_l_val;  /* 4 bytes */
 void interrupt (*oldfunc)(__CPPARGS); /* interrupt function pointer */

 FILE *stream;

 write_l_val = 1;

 /* open a file for update */
// stream = fopen("log", "w+");


   outpd ((word)(base_address_3 + PCIPAGE), 0);
	outpd ((word)(base_address_3 + PCIADDR), 0);
   outpd (base_address_1, 0);


 puts("Press <Shift><PrtSc> to terminate");

 /* save the old interrupt */
 oldfunc  = _dos_getvect(5);

 /* install interrupt handler */
 _dos_setvect(5,get_out);

 looping = 1;
 total_fails = 0;

 while (looping) {
   for ( j = 1 ; j < 9 ; j++ ){
      host_fill_memory_mem(j);

      outpd ((word)(base_address_3 + PCIPAGE), 0);
   	outpd ((word)(base_address_3 + PCIADDR), 8);
      outpd (base_address_1, j);             // parameter

     	outpd ((word)(base_address_3 + PCIADDR), 4);
      outpd (base_address_1, 1);             // command


     	outpd ((word)(base_address_3 + PCIADDR), 0);

      while( ((inpd( (word)(base_address_1)) & 0x1 )
           != 0x1) && looping  ) ;

      outpd ( (word)(base_address_1), 0);  /* clear memory mailbox */

     	outpd ((word)(base_address_3 + PCIADDR), 8);
      jb =  0xffff & inpd((word)(base_address_1));

      if ( fail = host_validate_memory_mem(jb) ){
	  	   printf ("test fails, number of failures = %d\n", fail);
//  	  	   fprintf (stream,"test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
	 	   printf ("test passes, errors(%d)\n", total_fails);
//	 	   fprintf (stream,"test passes, errors(%d)\n", total_fails);
         }
      }
 }  /* while (looping) */

 /* restore to original interrupt routine */
 _dos_setvect(5,oldfunc);

 /* close the file */
// fclose(stream);

}


void host_tf_test()
{
 long i;     // 4 bytes
 char key;
 int j, jb;
 int fail;
 long total_fails;
 unsigned long read_l_val;  /* 4 bytes */
 unsigned long write_l_val;    /* 4 bytes */

 outpd ((word)(base_address_3 + PCIPAGE), 0);
 outpd ((word)(base_address_3 + PCIADDR), 0);
 outpd (base_address_1, 0);

 looping = 1;
 total_fails = 0;

 while (looping) {
   for ( j = 1 ; j < 9 ; j++ ){

      srand ( j );
      outpd ((word)(base_address_3 + PCIPAGE), 1);

  		outpd ((word)(base_address_3 + PCIADDR), 0 );
		write_l_val =  (unsigned long)(0xff & rand());
		write_l_val += (unsigned long)(0xff & rand()) << 8;
		write_l_val += (unsigned long)(0xff & rand()) << 16;
		write_l_val += (unsigned long)(0xff & rand()) << 24;
		outpd (base_address_1, write_l_val);

      outpd ((word)(base_address_3 + PCIPAGE), 0);
   	outpd ((word)(base_address_3 + PCIADDR), 8);
      outpd (base_address_1, j);             // parameter

     	outpd ((word)(base_address_3 + PCIADDR), 4);
      outpd (base_address_1, 1);             // command

     	outpd ((word)(base_address_3 + PCIADDR), 0);

      while( ((inpd( (word)(base_address_1)) & 0x1 )
           != 0x1) && looping  ) ;

      outpd ( (word)(base_address_1), 0);  /* clear memory mailbox */

     	outpd ((word)(base_address_3 + PCIADDR), 8);
      jb =  0xffff & inpd((word)(base_address_1));


       fail = 0;

       /* read back the random word */
       srand(jb);
 	   outpd ((word)(base_address_3 + PCIPAGE), 1);

 		outpd ((word)(base_address_3 + PCIADDR), 0 );
 		write_l_val =  (unsigned long)(0xff & rand());
 		write_l_val += (unsigned long)(0xff & rand()) << 8;
 		write_l_val += (unsigned long)(0xff & rand()) << 16;
 		write_l_val += (unsigned long)(0xff & rand()) << 24;
 		read_l_val = inpd(base_address_1);

 		if ( (unsigned long) read_l_val != (unsigned long) write_l_val )
         fail  = fail + 1;

      if ( fail ){
	  	   printf ("test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
	 	   printf ("test passes, errors(%d)\n", total_fails);
         }
    }
 }  /* while (looping) */

}



void host_tfmb_test()
{
 long i;     // 4 bytes
 char key;
 int j, jb;
 int fail;
 long total_fails;
 unsigned long read_l_val;  /* 4 bytes */
 unsigned long write_l_val;    /* 4 bytes */


 inpd( (base_address_0 + IMB1) );  // flush

 looping = 1;
 total_fails = 0;

 while (looping) {
   for ( j = 1 ; j < 9 ; j++ ){

      srand ( j );
      outpd ((word)(base_address_3 + PCIPAGE), 1);
  		outpd ((word)(base_address_3 + PCIADDR), 0 );
		write_l_val =  (unsigned long)(0xff & rand());
		write_l_val += (unsigned long)(0xff & rand()) << 8;
		write_l_val += (unsigned long)(0xff & rand()) << 16;
		write_l_val += (unsigned long)(0xff & rand()) << 24;
		outpd (base_address_1, write_l_val);
      read_l_val = inpd(base_address_1);

      printf ("j (%d) read_l_val (%lx)!= write_l_val (%lx)\n",
           j, read_l_val, write_l_val );

      outpd ( base_address_0 + OMB1, j );
                             // write command

      while( ((inpd( (word)(base_address_0 + MBEF)) & 0x00030000 )
           != 0x00030000) && looping  ) ;
                  /* loop till
                     incoming mailbox IMB1 (low 2 bytes) are full */

      jb =  0xffff & inpd((word)(base_address_0 + IMB1));

       fail = 0;

       /* read back the random word */
       srand(jb);
 	   outpd ((word)(base_address_3 + PCIPAGE), 1);

 		outpd ((word)(base_address_3 + PCIADDR), 0 );
 		write_l_val =  (unsigned long)(0xff & rand());
 		write_l_val += (unsigned long)(0xff & rand()) << 8;
 		write_l_val += (unsigned long)(0xff & rand()) << 16;
 		write_l_val += (unsigned long)(0xff & rand()) << 24;
 		read_l_val = inpd(base_address_1);

 		if ( (unsigned long) read_l_val != (unsigned long) write_l_val )
         fail  = fail + 1;

      if ( fail ){
	  	   printf ("test fails, number of failures = %d\n", fail);
         total_fails++;
         }
	   else {
	 	   printf ("test passes, errors(%d)\n", total_fails);
         }
    }
 }  /* while (looping) */

}


void host_read_write_loop()
{
 int j;
 unsigned long read_l_val;  /* 4 bytes */
 unsigned long write_l_val;    /* 4 bytes */

 while ( 1 ) {
   for ( j = 1 ; j < 9 ; j++ ){

      srand ( j );
      outpd ((word)(base_address_3 + PCIPAGE), 1);
  		outpd ((word)(base_address_3 + PCIADDR), 0 );

		write_l_val =  (unsigned long)(0xff & rand());
		write_l_val += (unsigned long)(0xff & rand()) << 8;
		write_l_val += (unsigned long)(0xff & rand()) << 16;
		write_l_val += (unsigned long)(0xff & rand()) << 24;

		outpd (base_address_1, write_l_val);
      read_l_val = inpd(base_address_1);

      printf ("j (%d) read_l_val (%lx)!= write_l_val (%lx)\n",
           j, read_l_val, write_l_val );
      if ( read_l_val != write_l_val ) printf ("fail\n");

      }
   }
}


void host_walking_bits()
{
  char key;
  int fail;
  unsigned long read_l_val;     /* 4 bytes */
  unsigned long write_l_val;    /* 4 bytes */
  int bit_place;

  printf ("Memory(PCI) Walking 1's and 0's\n");
  write_l_val = 1;
  fail = 0;

  outpd ((word)(base_address_3 + PCIPAGE), 1);
  outpd ((word)(base_address_3 + PCIADDR), 0 );


  /* walking 1's and 0's*/

  printf ("   Write        Read           Write        Read\n");
  printf ("   Value        Value          Value        Value\n\n");

  for ( bit_place = 0; bit_place < 16; bit_place++ ) {
     outpd (base_address_1, write_l_val);
     read_l_val = inpd(base_address_1);
     if (  (unsigned long) (0xffffffffL & read_l_val) !=
	      (unsigned long) (0xffffffffL & write_l_val)  )
	  fail = fail + 1;;


     printf ("%12lx%12lx    ", 0xffffffffL & write_l_val,
		   0xffffffffL & read_l_val );
     outpd (base_address_1, ~write_l_val);
     read_l_val = inpd(base_address_1);
     if ( (unsigned long) (0xffffffffL & read_l_val) !=
	       (unsigned long) (0xffffffffL & ~write_l_val) )
	 fail  = fail + 1;

     printf ("%12lx%12lx\n", 0xffffffffL & ~write_l_val,
		 0xffffffffL & read_l_val );

     write_l_val = write_l_val * 2;
     }


     if ( fail )
       printf ("first half of test fails, number of failures = %d\n", fail);
     else
       printf ("first half of test passes\n");

  printf ("type ENTER key to continue\n");
  scanf ( "%c", &key );

  printf ("   Write        Read           Write        Read\n");
  printf ("   Value        Value          Value        Value\n\n");


  for ( bit_place = 0; bit_place < 16; bit_place++ ) {
     outpd (base_address_1, write_l_val);
     read_l_val = inpd(base_address_1);
     if (  (unsigned long) (0xffffffffL & read_l_val) !=
	      (unsigned long) (0xffffffffL & write_l_val)  )
	  fail = fail + 1;;


     printf ("%12lx%12lx    ", 0xffffffffL & write_l_val,
		   0xffffffffL & read_l_val );
     outpd (base_address_1, ~write_l_val);
     read_l_val = inpd(base_address_1);
     if ( (unsigned long) (0xffffffffL & read_l_val) !=
	       (unsigned long) (0xffffffffL & ~write_l_val) )
	 fail  = fail + 1;

     printf ("%12lx%12lx\n", 0xffffffffL & ~write_l_val,
		 0xffffffffL & read_l_val );

     write_l_val = write_l_val * 2;
     }

     if ( fail )
       printf ("test fails, number of failures = %d\n", fail);
     else
       printf ("test passes\n");

printf ("type ENTER key to continue\n");
scanf ( "%c", &key );

}

void host_memory_short_test()
{
 int  i;
 int  fail;
 char key;
 unsigned long read_l_val;     /* 4 bytes */
 unsigned long write_l_val;    /* 4 bytes */

 printf ("WRITE/VALIDATE Memory(PCI)\n");
 fail = 0;

 /* write 4 random words */
 srand(1);
 outpd (base_address_3 + PCIPAGE, 0x00);

 for ( i = 0; i < 5; i++ ){
     outpd (base_address_3 + PCIADDR, 4 * i );
     write_l_val =  (unsigned long)(0xff & rand());
     write_l_val += (unsigned long)(0xff & rand()) << 8;
     write_l_val += (unsigned long)(0xff & rand()) << 16;
     write_l_val += (unsigned long)(0xff & rand()) << 24;
     outpd (base_address_1, write_l_val);
     }

 /* read back the 4 random words */
 srand(1);
 printf ("   Write    Read\n");
 printf ("   Value    Value\n\n");

 for ( i = 0; i < 5; i++ ){
   outpd (base_address_3 + PCIADDR, 4 * i );
   write_l_val =  (unsigned long)(0xff & rand());
   write_l_val += (unsigned long)(0xff & rand()) << 8;
   write_l_val += (unsigned long)(0xff & rand()) << 16;
   write_l_val += (unsigned long)(0xff & rand()) << 24;
   read_l_val = inpd(base_address_1);
   if ( (unsigned long) read_l_val != (unsigned long) write_l_val )
      fail  = fail + 1;
   printf ("%10lx%10lx\n", write_l_val, read_l_val );
   }


 if ( fail )
  printf ("test fails, number of failures = %d\n", fail);
 else
  printf ("test passes\n");


 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );

}
