#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>
#include "amcc.h"

extern dword base_address_0;   /* PCI_CS_BASE_ADDRESS_0 - amcc */
extern dword base_address_1;   /* PCI_CS_BASE_ADDRESS_1 - memory */
extern dword base_address_2;   /* PCI_CS_BASE_ADDRESS_2 - pio */
extern dword base_address_3;   /* PCI_CS_BASE_ADDRESS_3 - cpld */
extern dword base_address_4;   /* PCI_CS_BASE_ADDRESS_4 - tmr */

void host_ramp_test();

/* ---------------------------------- */
/*      DSP register offsets          */
/* ---------------------------------- */

/* PCI Bus Operation Registers */

#define OMB1    0x00    /* outgoing mailbox register 1 */
#define OMB2    0x04    /* outgoing mailbox register 2 */
#define OMB3    0x08    /* outgoing mailbox register 3 */
#define OMB4    0x0c    /* outgoing mailbox register 4 */
#define IMB1    0x10    /* incoming mailbox register 1 */
#define IMB2    0x14    /* incoming mailbox register 2 */
#define IMB3    0x18    /* incoming mailbox register 3 */
#define IMB4    0x1c    /* incoming mailbox register 4 */
#define MBEF    0x34    /* mail-box empty/full status reg */

#define PCICMD  0x08
#define PCIADDR 0x04
#define PCIPAGE 0x00


/* --------------------------------------------- */
/* host_ramp_test ()                             */
/*                                               */
/* PAGE 1    put stream saw tooth                */
/* PAGE 2    put stream saw tooth (copy)         */
/* PAGE 3    get stream                          */
/* PAGE 4    get stream (copy)                   */
/*                                               */
/* ERROR 1   put stream pag1 did not validate    */
/* ERROR 2   put stream page 1 & 2 differ        */
/* ERROR 3   get stream page 3 & 4 differ        */
/* --------------------------------------------- */

void host_ramp_test()
{
 unsigned get_stream_even [ 1024 ];
 unsigned get_stream_odd  [ 1024 ];
 unsigned put_val_even;
 unsigned put_val_odd;
 unsigned get_val_even;
 unsigned get_val_odd;

 unsigned i;
 unsigned v;
 char key;
 int j;
 int keep_looping;
 int range;
 int loop_count;
 int error;
 unsigned long write_l_val;
 unsigned long read_l_val;
 FILE *stream;
 FILE *logstream;
 FILE *errstream;			// stress test log file

 inpd( (base_address_0 + IMB1) );  // flush

 /* open log file */
 logstream = fopen("log.txt", "w+");

 /* stress test, open error file */
 errstream = fopen("err.txt", "w+");

 keep_looping = 1;
 loop_count = 1;

 /* scope test pins, clear */
 outpd ((word)(base_address_3 + PCICMD), 0);

// stress test, add loop_count term to while

 while( keep_looping && loop_count < 3000 ){
// while( 1 ){
 error = 0;

 fprintf (logstream,"loop_count = %d\n", loop_count );

/* ------------------------------------------ */
/*  write put_stream to shared memory PAGE 1  */
/* ------------------------------------------ */

 outpd ((word)(base_address_3 + PCIPAGE), 1);

 /* move Sawtooth from zero to max positive value */

   v = 0;
	for ( j = 0; j < 256; j++ ){
	  outpd ((word)(base_address_3 + PCIADDR), 4 * j );
	  put_val_even =   v;
     put_val_odd  =   v + 64;
     write_l_val  =  ( (unsigned long) put_val_odd << 16 ) +
                        (unsigned long) put_val_even;
	  outpd (base_address_1, write_l_val);

// stress test, insert code
     read_l_val =  inpd (base_address_1);
     if ( write_l_val != read_l_val ){
         outpd ((word)(base_address_3 + PCICMD), 3);
         outpd ((word)(base_address_3 + PCICMD), 0);

         put_val_even  =  0xffff & ((unsigned) read_l_val);
         put_val_odd  = (unsigned) (0xffff & ( read_l_val >> 16));

         /* write to the file err.txt */
         fprintf (errstream,"loop_count = %d %d %d\n", loop_count, 2*j, 2*j+1 );
         fprintf (errstream,"%04x %04x  ", v,    put_val_even );
         fprintf (errstream,"%04x %04x\n", v+64, put_val_odd  );

         read_l_val =  inpd (base_address_1);
         put_val_even  =  0xffff & ((unsigned) read_l_val);
         put_val_odd  = (unsigned) (0xffff & ( read_l_val >> 16));

         /* write to the file err.txt */
         fprintf (errstream,"%04x %04x  ", v,    put_val_even );
         fprintf (errstream,"%04x %04x\n", v+64, put_val_odd  );

        }

// end insert code

     v = v + 128;
     }

   /* move Sawtooth from max positive to max negative value */
   v = 32704;
	for ( j = 256; j < 768; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		put_val_even =  v;
      put_val_odd  =  v-64;
      write_l_val  =  ( (unsigned long)put_val_odd << 16 ) +
                        (unsigned long)put_val_even;
		outpd (base_address_1, write_l_val);

      v=v-128;
		}

   /* move Sawtooth from max negative to 1 count less than zero */
   v = 32768;
	for ( j = 768; j < 1024; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		put_val_even = v;
      put_val_odd  =  v+64;
      write_l_val  =  ( (unsigned long)put_val_odd << 16 ) +
                        (unsigned long)put_val_even;
		outpd (base_address_1, write_l_val);

      v = v + 128;
		}

 /* validate values in put_stream and write to pstream.txt */

 /* open pstream.txt for update */
 stream = fopen("pstream.txt", "w+");
 fprintf (stream,"loop_count = %d\n", loop_count );

 outpd ((word)(base_address_3 + PCIPAGE), 1);

 /* move the Sawtooth from zero to max positive value */

   v = 0;
	for ( j = 0; j < 256; j++ ){
	  outpd ((word)(base_address_3 + PCIADDR), 4 * j );
	  put_val_even =   v;
     put_val_odd  =   v + 64;
     write_l_val  =  ( (unsigned long) put_val_odd << 16 ) +
                        (unsigned long) put_val_even;
     read_l_val   =  inpd ( base_address_1 );
     put_val_even  =  0xffff & ((unsigned) read_l_val);
     put_val_odd  = (unsigned) (0xffff & ( read_l_val >> 16));

     /* write to the file pstream.txt */
     fprintf (stream,"%05u\n", put_val_even );
     fprintf (stream,"%05u\n", put_val_odd  );

     if ( write_l_val != read_l_val ){
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
              2*j, put_val_even, v );
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
              j*2+1, put_val_odd, v + 64 );
         error = 1;
         }

      v = v + 128;
		}

   /* move the Sawtooth from max positive to max negative value */
   v = 32704;
	for ( j = 256; j < 768; j++ ){
	  outpd ((word)(base_address_3 + PCIADDR), 4 * j );
	  put_val_even =   v;
     put_val_odd  =   v - 64;
     write_l_val  =  ( (unsigned long) put_val_odd << 16 ) +
                        (unsigned long) put_val_even;
     read_l_val   =  inpd ( base_address_1 );
     put_val_even  =  0xffff & ((unsigned) read_l_val);
     put_val_odd  = (unsigned) (0xffff & ( read_l_val >> 16));

     /* write to the file pstream.txt */
     fprintf (stream,"%05u\n", put_val_even );
     fprintf (stream,"%05u\n", put_val_odd  );

     if ( write_l_val != read_l_val ){
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
              2*j, put_val_even, v );
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
              j*2+1, put_val_odd, v + 64 );
         error = 1;
         }

      v=v-128;
		}

   /* move Sawtooth from max negative to 1 count less than zero */
   v = 32768;
	for ( j = 768; j < 1024; j++ ){
	  outpd ((word)(base_address_3 + PCIADDR), 4 * j );
	  put_val_even =   v;
     put_val_odd  =   v + 64;
     write_l_val  =  ( (unsigned long) put_val_odd << 16 ) +
                        (unsigned long) put_val_even;
     read_l_val   =  inpd ( base_address_1 );
     put_val_even  =  0xffff & ((unsigned) read_l_val);
     put_val_odd  = (unsigned) (0xffff & ( read_l_val >> 16));

     /* write to the file pstream.txt */
     fprintf (stream,"%05u\n", put_val_even );
     fprintf (stream,"%05u\n", put_val_odd  );

     if ( write_l_val != read_l_val ){
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
              2*j, put_val_even, v );
         fprintf (logstream,"put_stream [ %d ] = %d should be = %d\n",
              j*2+1, put_val_odd, v + 64 );
         error = 1;
         }

      v = v + 128;
		}



 /* close the file */
 fclose(stream);

 /* write copy of put_stream to shared memory PAGE 2 */
 outpd ((word)(base_address_3 + PCIPAGE), 2);

 /* move Sawtooth from zero to max positive value */

   v = 0;
	for ( j = 0; j < 256; j++ ){
	  outpd ((word)(base_address_3 + PCIADDR), 4 * j );
	  put_val_even =   v;
     put_val_odd  =   v + 64;
     write_l_val  =  ( (unsigned long) put_val_odd << 16 ) +
                        (unsigned long) put_val_even;
	  outpd (base_address_1, write_l_val);

     v = v + 128;
     }

   /* move Sawtooth from max positive to max negative value */
   v = 32704;
	for ( j = 256; j < 768; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		put_val_even =  v;
      put_val_odd  =  v-64;
      write_l_val  =  ( (unsigned long)put_val_odd << 16 ) +
                        (unsigned long)put_val_even;
		outpd (base_address_1, write_l_val);

      v=v-128;
		}

   /* move Sawtooth from max negative to 1 count less than zero */
   v = 32768;
	for ( j = 768; j < 1024; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		put_val_even = v;
      put_val_odd  =  v+64;
      write_l_val  =  ( (unsigned long)put_val_odd << 16 ) +
                        (unsigned long)put_val_even;
		outpd (base_address_1, write_l_val);

      v = v + 128;
		}

/* -------------- */
/*  SEND COMMAND  */
/* -------------- */

 outpd ( base_address_0 + OMB1, j );
                      // write command


/* --------------- */
/*  WAIT FOR DONE  */
/* --------------- */

 printf("%d wait for control ... ", loop_count );

 while( (inpd( (word)(base_address_0 + MBEF)) & 0x00030000 )
           != 0x00030000   ) ;
                  /* loop till
                     incoming mailbox IMB1 (low 2 bytes) are full */

 printf("received done message\n");

 inpd( (base_address_0 + IMB1) );  // flush


  /* read get_stream from shared memory, PAGE 3 */
  outpd ((word)(base_address_3 + PCIPAGE), 3);

  for ( j = 0; j < 1024; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		get_stream_even [ j ] = 0xffff & inpd (base_address_1);
		get_stream_odd [ j ]  = 0xffff & ( inpd (base_address_1) >> 16 );
		}

 /* open gstream.txt for update */
 stream = fopen("gstream.txt", "w+");

 for ( j = 0; j < 1024; j++ ){
   fprintf (stream,"%05u\n", get_stream_even[ j ] );
   fprintf (stream,"%05u\n", get_stream_odd[ j ] );
 }

 /* close the file */
 fclose(stream);

 /* open getval.txt for update */
 stream = fopen("getval.txt", "w+");

 for ( j = 0; j < 1024; j++ ){
   fprintf (stream,"%05d\n", (int) get_stream_even[ j ] );
   fprintf (stream,"%05d\n", (int) get_stream_odd[ j ] );
 }

 /* close the file */
 fclose(stream);

 /* ----------------------------------------------- */
 /*    check for out of range values in getstream   */
 /* ----------------------------------------------- */

 /* open range.txt for update */
 stream = fopen("range.txt", "w+");

 /* move Sawtooth from zero to max positive value */

   v = 0;
	for ( j = 0; j < 256; j++ ){
		put_val_even = v;
      put_val_odd  =  (v+64);

      range = (int) get_stream_even [ j ] - (int) put_val_even;
      if ( (range > 1500) || (range < -1500) ){
         error = 2;
         fprintf (logstream,
           "out of range val, get_stream[%d] = %u error(%d)\n",
           2*j, get_stream_even[ j ], error );
         }

      fprintf (stream,"%d\n", range );

      range = (int) get_stream_odd [ j ] - (int) put_val_odd;
      if ( (range > 1500) || (range < -1500) ){
         error = 2;
         fprintf (logstream,
           "out of range val, get_stream[%d] = %u error(%d)\n",
           2*j+1, get_stream_odd[ j ], error );
         }

      fprintf (stream,"%d\n", range );

      v = v+128;
		}

   /* move Sawtooth from max positive to max negative value */
   v = 32704;
	for ( j = 256; j < 768; j++ ){
		put_val_even =  v;
      put_val_odd  =  v - 64;

      range = (int) get_stream_even [ j ] - (int) put_val_even;
      if ( (range > 1500) || (range < -1500) ){
         error = 2;
         fprintf (logstream,
           "out of range val, get_stream[%d] = %u error(%d)\n",
           2*j, get_stream_even[ j ], error );
         }

      fprintf (stream,"%d\n", range );

      range = (int) get_stream_odd [ j ] - (int) put_val_odd;
      if ( (range > 1500) || (range < -1500) ){
         error = 2;
         fprintf (logstream,
           "out of range val, get_stream[%d] = %u error(%d)\n",
           2*j+1, get_stream_odd[ j ], error );
         }

      fprintf (stream,"%d\n", range );

      v = v - 128;
		}

   /* move Sawtooth from max negative to 1 count less than zero */
   v = 32768;
	for ( j = 768; j < 1024; j++ ){
		put_val_even = v;
      put_val_odd  =  v + 64;

      range = (int) get_stream_even [ j ] - (int) put_val_even;
      if ( (range > 1500) || (range < -1500) ){
         error = 2;
         fprintf (logstream,
           "out of range val, get_stream[%d] = %u error(%d)\n",
           2*j, get_stream_even[ j ], error );
         }

      fprintf (stream,"%d\n", range );

      range = (int) get_stream_odd [ j ] - (int) put_val_odd;
      if ( (range > 1500) || (range < -1500) ){
         error = 2;
         fprintf (logstream,
           "out of range val, get_stream[%d] = %u error(%d)\n",
           2*j+1, get_stream_odd[ j ], error );
         }

      fprintf (stream,"%d\n", range );

      v = v + 128;
		}

 /* close the file */
 fclose(stream);

/* ----------------------------------- */
/*  check that they are identical      */
/*    get stream in page 3             */
/*    copy of get stream in page 4     */
/* ----------------------------------- */

  outpd ((word)(base_address_3 + PCIPAGE), 4);

  for ( j = 0; j < 1024; j++ ){
		outpd ((word)(base_address_3 + PCIADDR), 4 * j );
		get_val_even  = 0xffff & inpd (base_address_1);

		if ( get_stream_even [ j ] != get_val_even ){
              error = 3;
              fprintf (logstream,
              "get_stream [ %d ]  Page 3 = %u Page 4 = %u error(%d)\n",
                    2 * j, get_stream_even [ j ], get_val_even, error );
              }
		get_val_odd  = 0xffff & ( inpd (base_address_1) >> 16 );
		if ( get_stream_odd  [ j ] != get_val_odd ){
              error = 3;
              fprintf (logstream,
              "get_stream [ %d ]  Page 3 = %u Page 4 = %u error(%d)\n",
                    2 * j + 1, get_stream_odd [ j ], get_val_odd, error );
              }
		}

// stress test, comment out

// if( error ) keep_looping = 0;
 loop_count++;

 }  // while(keep_looping)

 fclose(logstream);

/* stress test */
 fclose(errstream);
 }
