#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>
#include "amcc.h"

void validate_incoming_mailbox();
void write_outgoing_mailbox();
void host_mailbox_walking_bits();

extern dword base_address_0;   /* PCI_CS_BASE_ADDRESS_0 - amcc */
extern dword base_address_1;   /* PCI_CS_BASE_ADDRESS_1 - memory */
extern dword base_address_2;   /* PCI_CS_BASE_ADDRESS_2 - pio */
extern dword base_address_3;   /* PCI_CS_BASE_ADDRESS_3 - cpld */
extern dword base_address_4;   /* PCI_CS_BASE_ADDRESS_4 - tmr */

char read_c_val;

/* ---------------------------------- */
/*      DSP register offsets          */
/* ---------------------------------- */

/* PCI Bus Operation Registers */

#define OMB1    0x00    /* outgoing mailbox register 1 */
#define OMB2    0x04    /* outgoing mailbox register 2 */
#define OMB3    0x08    /* outgoing mailbox register 3 */
#define OMB4    0x0c    /* outgoing mailbox register 4 */
#define IMB1    0x10    /* incoming mailbox register 1 */
#define IMB2    0x14    /* incoming mailbox register 2 */
#define IMB3    0x18    /* incoming mailbox register 3 */
#define IMB4    0x1c    /* incoming mailbox register 4 */
#define MBEF    0x34    /* mail-box empty/full status reg */

#define PCICMD  0x08
#define PCIADDR 0x04
#define PCIPAGE 0x00

void validate_incoming_mailbox()
{
 unsigned long imb1_val;    /* 4 bytes */
 unsigned long imb2_val;
 unsigned long imb3_val;
 unsigned long imb4_val;

 unsigned long aomb1_val;    /* 4 bytes */
 unsigned long aomb2_val;
 unsigned long aomb3_val;
 unsigned long aomb4_val;

 char key;

 inpd( (base_address_0 + IMB1) );  // flush
 inpd( (base_address_0 + IMB2) );
 inpd( (base_address_0 + IMB3) );
 inpd( (base_address_0 + IMB4) );

 while( (inpd( (word)(base_address_0 + MBEF)) & 0x000f0000L )
           !=  0x000f0000L   ) ;
                  /* loop till incoming mailbox IMB1
                     (low 4 bytes) are full */

 imb1_val =  0xffffffffL & inpd((word)(base_address_0 + IMB1));
 imb2_val =  0xffffffffL & inpd((word)(base_address_0 + IMB2));
 imb3_val =  0xffffffffL & inpd((word)(base_address_0 + IMB3));
 imb4_val =  0x00ffffffL & inpd((word)(base_address_0 + IMB4));

 /* recreate the values written by the control computer to
    addon outgoing mailbox registers AOMB1..AOMB4           */

 srand(16);

 aomb1_val =  (unsigned long)(0xff & rand());
 aomb1_val += (unsigned long)(0xff & rand()) << 8;
 aomb1_val += (unsigned long)(0xff & rand()) << 16;
 aomb1_val += (unsigned long)(0xff & rand()) << 24;

 aomb2_val =  (unsigned long)(0xff & rand());
 aomb2_val += (unsigned long)(0xff & rand()) << 8;
 aomb2_val += (unsigned long)(0xff & rand()) << 16;
 aomb2_val += (unsigned long)(0xff & rand()) << 24;

 aomb3_val =  (unsigned long)(0xff & rand());
 aomb3_val += (unsigned long)(0xff & rand()) << 8;
 aomb3_val += (unsigned long)(0xff & rand()) << 16;
 aomb3_val += (unsigned long)(0xff & rand()) << 24;

 aomb4_val =  (unsigned long)(0xff & rand());
 aomb4_val += (unsigned long)(0xff & rand()) << 8;
 aomb4_val += (unsigned long)(0xff & rand()) << 16;
 /* note: aomb4 does not have bits 24 to 31 */

 /* Do the values match ?
    AOMB1..AOMB4 values written by the control computer
    IMB1 ..IMB4  values read by the host computer       */

 printf ("Verify that the values IMB1..IMB4 match AOMB1..AOMB4\n");
 printf (
 "  AOMB1..AOMB4 addon out mailbox regs. written to by the control computer\n");
 printf (
 "  IMB1..IMB4 input mailbox registers read by the host computer\n");

 printf("\nAOMB1 = 0x%08lx  IMB1 = 0x%08lx   ", aomb1_val, imb1_val);
 if(aomb1_val == imb1_val)
   printf("values match\n");
 else
   printf("values do not match\n");

 printf("\nAOMB2 = 0x%08lx  IMB2 = 0x%08lx   ", aomb2_val, imb2_val);
 if(aomb2_val == imb2_val)
   printf("values match\n");
 else
   printf("values do not match\n");

 printf("\nAOMB3 = 0x%08lx  IMB3 = 0x%08lx   ", aomb3_val, imb3_val);
 if(aomb3_val == imb3_val)
   printf("values match\n");
 else
   printf("values do not match\n");

 printf("\nAOMB4 = 0x%08lx  IMB4 = 0x%08lx   ", aomb4_val, imb4_val);
 if(aomb4_val == imb4_val)
   printf("values match\n");
 else
   printf("values do not match\n");


 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );

}

/* ------------------------------------------------------------------ */
/*  void write_outgoing_mailbox()                                     */
/*  write to OMB1..OMB4 and then verify that                          */
/*  these values are read by the control computer                     */
/*  in AIMB1..AIMB4                                                   */
/*  OMB1..OMB4 Outgoing Mailbox regs ( written to by host )           */
/*  AIMB1..AIMB4 AddOn incoming mailbox regs (read by ctrl computer ) */
/* ------------------------------------------------------------------ */

void write_outgoing_mailbox()
{
 unsigned long omb1_val;    /* 4 bytes */
 unsigned long omb2_val;
 unsigned long omb3_val;
 unsigned long omb4_val;

 char key;

 printf("Write to OMB1 .. OMB4  ....");


 /* Generate random values for OMB1 .. OMB4 */
 srand(14);

 omb1_val =  (unsigned long)(0xff & rand());
 omb1_val += (unsigned long)(0xff & rand()) << 8;
 omb1_val += (unsigned long)(0xff & rand()) << 16;
 omb1_val += (unsigned long)(0xff & rand()) << 24;

 omb2_val =  (unsigned long)(0xff & rand());
 omb2_val += (unsigned long)(0xff & rand()) << 8;
 omb2_val += (unsigned long)(0xff & rand()) << 16;
 omb2_val += (unsigned long)(0xff & rand()) << 24;

 omb3_val =  (unsigned long)(0xff & rand());
 omb3_val += (unsigned long)(0xff & rand()) << 8;
 omb3_val += (unsigned long)(0xff & rand()) << 16;
 omb3_val += (unsigned long)(0xff & rand()) << 24;

 omb4_val =  (unsigned long)(0xff & rand());
 omb4_val += (unsigned long)(0xff & rand()) << 8;
 omb4_val += (unsigned long)(0xff & rand()) << 16;
 omb4_val += (unsigned long)(0xff & rand()) << 24;

 outpd (base_address_0 + OMB4, omb4_val );
 outpd (base_address_0 + OMB3, omb3_val );
 outpd (base_address_0 + OMB2, omb2_val );
 outpd (base_address_0 + OMB1, omb1_val );

 printf(" done \n");
 printf ("type ENTER key to continue\n");
 scanf ( "%c", &key );
}


/* ----------------------------------------------- */
/*    Write to PCI Outgoing Mailbox Register OMB1  */
/* ----------------------------------------------- */

void host_mailbox_walking_bits()
{
  char key;
  int fail;
  unsigned long read_l_val;     /* 4 bytes */
  unsigned long write_l_val;    /* 4 bytes */
  int bit_place;


  printf ("OMB1 Write / Read Back\n");


  write_l_val = 1;
  fail = 0;

  /* walking 1's and 0's*/

  printf ("   Write        Read           Write        Read\n");
  printf ("   Value        Value          Value        Value\n\n");

  for ( bit_place = 0; bit_place < 16; bit_place++ ) {
     outpd (base_address_0 + OMB1, write_l_val );
     read_l_val = inpd(base_address_0 + OMB1);
     if (  (unsigned long) (0xffffffffL & read_l_val) !=
	      (unsigned long) (0xffffffffL & write_l_val)  )
	  fail = fail + 1;;


     printf ("%12lx%12lx    ", 0xffffffffL & write_l_val,
		   0xffffffffL & read_l_val );

     outpd (base_address_0 + OMB1, ~write_l_val);
     read_l_val = inpd(base_address_0 + OMB1);
     if ( (unsigned long) (0xffffffffL & read_l_val) !=
	       (unsigned long) (0xffffffffL & ~write_l_val) )
	 fail  = fail + 1;

     printf ("%12lx%12lx\n", 0xffffffffL & ~write_l_val,
		 0xffffffffL & read_l_val );

     write_l_val = write_l_val * 2;
     }


     if ( fail )
       printf ("first half of test fails, number of failures = %d\n", fail);
     else
       printf ("first half of test passes\n");

  printf ("type ENTER key to continue\n");
  scanf ( "%c", &key );

  printf ("   Write        Read           Write        Read\n");
  printf ("   Value        Value          Value        Value\n\n");


  for ( bit_place = 0; bit_place < 16; bit_place++ ) {
     outpd (base_address_0 + OMB1, write_l_val );
     read_l_val = inpd(base_address_0 + OMB1);
     if (  (unsigned long) (0xffffffffL & read_l_val) !=
	      (unsigned long) (0xffffffffL & write_l_val)  )
	  fail = fail + 1;;


     printf ("%12lx%12lx    ", 0xffffffffL & write_l_val,
		   0xffffffffL & read_l_val );

     outpd (base_address_0 + OMB1, ~write_l_val);
     read_l_val = inpd(base_address_0 + OMB1);
     if ( (unsigned long) (0xffffffffL & read_l_val) !=
	       (unsigned long) (0xffffffffL & ~write_l_val) )
	 fail  = fail + 1;

     printf ("%12lx%12lx\n", 0xffffffffL & ~write_l_val,
		 0xffffffffL & read_l_val );

     write_l_val = write_l_val * 2;
     }

     if ( fail )
       printf ("test fails, number of failures = %d\n", fail);
     else
       printf ("test passes\n");

  printf ("type ENTER key to continue\n");
  scanf ( "%c", &key );
}
